import os, sys, shutil, platform
from pathlib import Path
import importlib.resources as res

APP_NAME = "Gel Annotate"

def _icon_file(ext: str) -> str:
    name = f"icon.{ext}"
    with res.as_file(res.files("annotation.assets") / name) as p:
        return str(p)

def _is_windows() -> bool: return platform.system().lower().startswith("win")
def _is_macos() -> bool:   return platform.system().lower().startswith("darwin")
def _is_linux() -> bool:   return platform.system().lower().startswith("linux")

def create_windows_shortcuts():
    try:
        import win32com.client  # from pywin32
    except Exception as e:
        print("pywin32 is required on Windows: pip install aer-gel-annotate[windows]")
        raise

    shell = win32com.client.Dispatch("WScript.Shell")

    # Where to place shortcuts
    desktop = Path(os.path.join(os.path.expanduser("~"), "Desktop"))
    start_menu = Path(os.path.join(os.environ.get("APPDATA", ""), "Microsoft", "Windows", "Start Menu", "Programs"))

    target = sys.executable
    args = "-m annotation.gui.gel_gui_wrapper"   # use module execution so venv is respected
    icon = _icon_file("ico")
    workdir = str(Path.home())

    for dest_dir in [desktop, start_menu]:
        dest_dir.mkdir(parents=True, exist_ok=True)
        lnk = str(dest_dir / f"{APP_NAME}.lnk")
        sc = shell.CreateShortCut(lnk)
        sc.Targetpath = target
        sc.Arguments = args
        sc.IconLocation = icon
        sc.WorkingDirectory = workdir
        sc.Description = APP_NAME
        sc.save()
        print(f"Created: {lnk}")

def create_linux_shortcut():
    applications_dir = Path("~/.local/share/applications").expanduser()
    desktop_dir = Path("~/Desktop").expanduser()
    applications_dir.mkdir(parents=True, exist_ok=True)

    icon_src = _icon_file("png")
    # Copy icon into a stable place in ~/.local/share/icons
    icon_target_dir = Path("~/.local/share/icons").expanduser()
    icon_target_dir.mkdir(parents=True, exist_ok=True)
    icon_target = icon_target_dir / "aer-gel-annotate.png"
    shutil.copyfile(icon_src, icon_target)

    desktop_entry = f"""[Desktop Entry]
Type=Application
Name={APP_NAME}
Comment=Gel well detection and annotation
Exec=gel-annotate
Icon={icon_target}
Terminal=false
Categories=Education;Science;
"""
    appfile = applications_dir / "aer-gel-annotate.desktop"
    appfile.write_text(desktop_entry)
    print(f"Created: {appfile}")

    # Also drop a desktop shortcut if Desktop exists
    if desktop_dir.exists():
        desktop_file = desktop_dir / f"{APP_NAME}.desktop"
        desktop_file.write_text(desktop_entry)
        desktop_file.chmod(0o755)
        print(f"Created: {desktop_file}")

def create_macos_shortcut():
    # Simple .command launcher on Desktop (double-clickable)
    desktop = Path("~/Desktop").expanduser()
    desktop.mkdir(parents=True, exist_ok=True)
    cmd = desktop / f"{APP_NAME}.command"
    cmd.write_text("#!/bin/bash\nexec gel-annotate\n")
    cmd.chmod(0o755)
    print(f"Created: {cmd}")
    print("Tip: drag the shortcut into the Dock if you want quick access.")
    # (Attaching a custom icon to a .command file is manual via Finder → Get Info.)

def main():
    if _is_windows():
        create_windows_shortcuts()
    elif _is_linux():
        create_linux_shortcut()
    elif _is_macos():
        create_macos_shortcut()
    else:
        print("Unsupported OS?")
        sys.exit(1)

if __name__ == "__main__":
    main()



import numpy as np
import matplotlib.pyplot as plt

def detect_pcr_bands(img_corrected, wells, max_width=100, min_row_spacing=115,
                     ax=None, return_results=False,
                     gradient_thresh=0.05, min_prominence=0.02, smoothing_sigma=1,
                     manual_yes=None, manual_no=None,
                     draw_trace_on_image=False, use_smoothed=True):
    if manual_no is None:
        manual_no = []
    if manual_yes is None:
        manual_yes = []

    from scipy.ndimage import gaussian_filter1d
    from scipy.signal import find_peaks

    if ax is None:
        fig, ax = plt.subplots(figsize=(15, 10))
        ax.imshow(img_corrected, cmap='gray')

    results = []

    for idx, (x, y) in enumerate(wells, start=1):
        x1 = int(x - max_width / 2)
        x2 = int(x + max_width / 2)
        y1 = int(y + 5)
        y2 = int(y + min_row_spacing)

        x1 = max(0, x1); x2 = min(img_corrected.shape[1], x2)
        y1 = max(0, y1); y2 = min(img_corrected.shape[0], y2)

        narrow_x1 = max(x1, int(x - 5))
        narrow_x2 = min(x2, int(x + 5))

        strip = img_corrected[y1:y2, narrow_x1:narrow_x2]
        vertical_profile = strip.mean(axis=1)

        smoothed = gaussian_filter1d(vertical_profile, sigma=smoothing_sigma)
        trace = smoothed if use_smoothed else vertical_profile

        grad = np.gradient(trace)
        peaks, _ = find_peaks(grad, height=gradient_thresh, prominence=min_prominence)
        has_band = len(peaks) > 0
        results.append(has_band)

        if manual_yes and idx in manual_yes:
            color = 'lime'
        elif manual_no and idx in manual_no:
            color = 'red'
        else:
            color = 'lime' if has_band else 'red'

        ax.add_patch(plt.Rectangle((x1, y1), x2 - x1, y2 - y1,
                                   edgecolor=color, facecolor='none', linewidth=1))
        if draw_trace_on_image:
            trace_norm = (trace - trace.min()) / (trace.max() - trace.min() + 1e-8)
            trace_scaled = x1 + trace_norm * (x2 - x1)
            y_range = np.arange(y1, y2)
            ax.plot(trace_scaled, y_range, color='blue', linewidth=1)
            for peak in peaks:
                peak_y = y1 + peak
                peak_x = trace_scaled[peak]
                ax.plot(peak_x, peak_y, 'ro', markersize=3)

    if return_results:
        return results



import numpy as np
from skimage import exposure, filters, morphology, measure, transform

def load_and_preprocess_image(img, *, gamma=1.0, target_width=700):
    # Resize
    scale = target_width / img.shape[1]
    new_height = int(img.shape[0] * scale)
    img_resized = transform.resize(img, (new_height, target_width), preserve_range=True, anti_aliasing=True)

    # Mild contrast enhance
    p2, p98 = np.percentile(img_resized, (2, 98))
    img_contrast = exposure.rescale_intensity(img_resized, in_range=(p2, p98))

    # Gamma correction
    img_corrected = exposure.adjust_gamma(img_contrast, gamma=gamma)

    # Inverse vignette
    rows, cols = img_corrected.shape
    Y, X = np.ogrid[:rows, :cols]
    center_y, center_x = rows / 2, cols / 2
    radius = np.sqrt((X - center_x) ** 2 + (Y - center_y) ** 2)
    max_radius = np.sqrt(center_x ** 2 + center_y ** 2)
    vignette = 1 + 0.4 * (1 - (radius / max_radius))
    vignette = np.clip(vignette, 1, 1.4)

    return np.clip(img_corrected / vignette, 0, 1)

def threshold_image(img_corrected, method="local"):
    if method == "local":
        block_size = 51
        offset = 0.01
        thr = filters.threshold_local(img_corrected, block_size=block_size, offset=offset)
        binary = img_corrected < thr
    elif method == "multiotsu":
        thr_vals = filters.threshold_multiotsu(img_corrected, classes=3)
        binary = img_corrected < thr_vals[0]
    elif method == "sauvola":
        thr = filters.threshold_sauvola(img_corrected, window_size=51)
        binary = img_corrected < thr
    elif method == "niblack":
        thr = filters.threshold_niblack(img_corrected, window_size=51)
        binary = img_corrected < thr
    else:
        raise ValueError("Unknown thresholding method")
    return binary

def clean_binary_image(binary):
    binary = morphology.binary_opening(binary, morphology.disk(1))
    cleaned = morphology.remove_small_objects(binary, min_size=5)
    cleaned = morphology.remove_small_holes(cleaned, area_threshold=5)
    return cleaned

def extract_valid_centroids(cleaned, img_corrected,
                            *, min_width=16, max_width=30, min_height=6, max_height=200,
                            max_mean_intensity=0.6):
    label_image = measure.label(cleaned)
    props = measure.regionprops(label_image, intensity_image=img_corrected)
    centroids = []
    for prop in props:
        minr, minc, maxr, maxc = prop.bbox
        h = maxr - minr
        w = maxc - minc
        if (min_width <= w <= max_width and
            min_height <= h <= max_height and
            prop.mean_intensity < max_mean_intensity):
            centroids.append(prop.centroid[::-1])  # (x, y)
    return centroids
